@@ -22,7 +22,7 @@
 *     [9] Cortes, David. "Imputing missing values with unsupervised random trees." arXiv preprint arXiv:1911.06646 (2019).
 * 
 *     BSD 2-Clause License
-*     Copyright (c) 2019, David Cortes
+*     Copyright (c) 2020, David Cortes
 *     All rights reserved.
 *     Redistribution and use in source and binary forms, with or without
 *     modification, are permitted provided that the following conditions are met:
@@ -287,6 +287,10 @@ bool interrupt_switch;
 *       'categ_data', and 'Xc', will get overwritten with the imputations produced.
 * - random_seed
 *       Seed that will be used to generate random numbers used by the model.
+* - handle_interrupt
+*       Whether to handle interrupt signals while the process is running. Note that this will
+*       interfere with interrupt handles when the procedure is called from interpreted languages
+*       such as Python or R.
 * - nthreads
 *       Number of parallel threads to use. Note that, the more threads, the more memory will be
 *       allocated, even if the thread does not end up being used. Ignored when not building with
@@ -337,7 +341,7 @@ int fit_iforest(IsoForest *model_outputs, ExtIsoForest *model_outputs_ext,
                 CategSplit cat_split_type, NewCategAction new_cat_action,
                 bool   all_perm, Imputer *imputer, size_t min_imp_obs,
                 UseDepthImp depth_imp, WeighImpRows weigh_imp_rows, bool impute_at_fit,
-                uint64_t random_seed, int nthreads)
+                uint64_t random_seed, bool handle_interrupt, int nthreads)
 {
     /* calculate maximum number of categories to use later */
     int max_categ = 0;
@@ -422,9 +426,12 @@ int fit_iforest(IsoForest *model_outputs, ExtIsoForest *model_outputs_ext,
        The following will still change the behavior of interrupts when called through e.g. Flask */
     #if !defined(_WIN32) && !defined(_WIN64) && !defined(_MSC_VER)
     struct sigaction sig_handle = {};
-    sig_handle.sa_flags = SA_RESETHAND;
-    sig_handle.sa_handler = set_interrup_global_variable;
-    sigemptyset(&sig_handle.sa_mask);
+    if (handle_interrupt)
+    {
+        sig_handle.sa_flags = SA_RESETHAND;
+        sig_handle.sa_handler = set_interrup_global_variable;
+        sigemptyset(&sig_handle.sa_mask);
+    }
     #endif
 
     /* grow trees */
@@ -469,11 +476,14 @@ int fit_iforest(IsoForest *model_outputs, ExtIsoForest *model_outputs_ext,
         else
             model_outputs_ext->hplanes[tree].shrink_to_fit();
 
-        #if !defined(_WIN32) && !defined(_WIN64) && !defined(_MSC_VER)
-        sigaction(SIGINT, &sig_handle, NULL);
-        #else
-        signal(SIGINT, set_interrup_global_variable);
-        #endif
+        if (handle_interrupt)
+        {
+            #if !defined(_WIN32) && !defined(_WIN64) && !defined(_MSC_VER)
+            sigaction(SIGINT, &sig_handle, NULL);
+            #else
+            signal(SIGINT, set_interrup_global_variable);
+            #endif
+        }
     }
 
     /* check if the procedure got interrupted */
